#include <ansi_c.h>
#include <visa.h>
#include <cvirte.h>     
#include <userint.h>
#include "RSSMU_3GPP_Example.h"
#include "rssmu.h"

/* Error check macro for driver functions */

#define CHECKERR( fCal ) \
    { \
        error = (fCal);\
        rssmu_error_message (io, error, error_message);\
        if (error < VI_SUCCESS) {\
            sprintf (error_buffer, "Primary Error: 0x%08X, %s\n", error, error_message);\
            MessagePopup ("Error", error_buffer); \
            rssmu_error_query (io, &error, error_message);\
            sprintf (error_buffer, "Secondary Error: 0x%08X, %s\n", error, error_message);\
            SetWaitCursor (0); \
            MessagePopup ("Error", error_buffer); \
            rssmu_close(io); \
            SetCtrlAttribute (panelHandle, PANEL_APPLY, ATTR_DIMMED, 0);\
            SetCtrlAttribute (panelHandle, PANEL_Exit, ATTR_DIMMED, 0);\
            break; \
        } \
    }
    
/* Globals */

ViInt32 panelHandle;
ViSession io;
ViStatus    error;
ViChar      error_message[256];
ViChar      error_buffer[256];

/* Local Functions */
int initialize (void);
            
/*= Main Function ===========================================================*/   
int main (int argc, char *argv[])
{
    if (InitCVIRTE (0, argv, 0) == 0)
        return -1;  /* out of memory */
    panelHandle = LoadPanel (0, "RSSMU_3GPP_Example.uir", PANEL); 
    DisplayPanel (panelHandle);
    RunUserInterface ();
    DiscardPanel (panelHandle);
    return 0;
}


/*===========================================================================*/
/* Function: Exit                                                            */
/* Purpose:  This is a callback function of Exit button. It closes the       */
/*           session and exits the example.                                  */
/*===========================================================================*/
int CVICALLBACK exitApp (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    switch (event)
        {
        case EVENT_COMMIT:
            QuitUserInterface (0);
            break;
        }
    return 0;
}

/*===========================================================================*/
/* Function: Panel Help                                                      */
/* Purpose:  This is a callback function of the menu bar. It displays a help */
/*           window describing panel being used.                             */
/*===========================================================================*/
void CVICALLBACK panelHelp (int menuBar, int menuItem, void *callbackData,
        int panel)
{

    if (panel == panelHandle)
        MessagePopup ("Help", "This is a simple demo program using\n"
                              "the Rohde&Schwarz SMU Vector Signal Generator\n"
                              "instrument driver provided in the package.\n");

    return;
}


/*===========================================================================*/
/* Function: Apply Settings                                                  */
/* Purpose:  This is a callback function of Apply button. The values are     */
/*           send to the instrument                                          */
/*===========================================================================*/
int CVICALLBACK apllySettings (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    ViInt32     model, seq_len, sync=0;
    ViBoolean   state;
    ViReal64    rf_level, rf_freq;
    
    switch (event)
    {
        case EVENT_COMMIT:

             SetCtrlAttribute (panelHandle, PANEL_APPLY, ATTR_DIMMED, 1);
             SetCtrlAttribute (panelHandle, PANEL_Exit, ATTR_DIMMED, 1);
             
             ProcessDrawEvents ();
             
             if (initialize() < VI_SUCCESS) break;

             SetWaitCursor (1);

             GetCtrlVal (panelHandle, PANEL_TEST_MODEL, &model); 
             GetCtrlVal (panelHandle, PANEL_SEQ_LENGTH, &seq_len);
             GetCtrlVal (panelHandle, PANEL_3GPP_STATE, &state);
             CHECKERR (rssmu_SetW3GPPTestModelBS (io, RSSMU_PATH_A, model));
             CHECKERR (rssmu_SetW3GPPSequenceLengthARB (io, RSSMU_PATH_A, seq_len));
             
             //For Syncronisation 
             //Enable Operation Complete for STB Register
             CHECKERR (rssmu_EventStatusEnable(io,1));
             //Clear Status
             CHECKERR (rssmu_ClearStatus(io));
             
             CHECKERR (rssmu_SetW3GPPFDDState (io, RSSMU_PATH_A, state));
             
             // Operation Complete?
             // send *OPC
              CHECKERR (rssmu_OperationComplete (io)); 
             // wait 
             do
             {
                Delay (0.1);
                CHECKERR (rssmu_StatusByteQuery(io,&sync));
             } while (!sync);
             
             GetCtrlVal (panelHandle, PANEL_RF_LEVEL, &rf_level);
             GetCtrlVal (panelHandle, PANEL_RF_FREQ, &rf_freq); 
             GetCtrlVal (panelHandle, PANEL_RF_STATE, &state);
             CHECKERR (rssmu_SetRFAmplitude (io, RSSMU_PATH_A, rf_level));
             CHECKERR (rssmu_SetRFFrequency (io, RSSMU_PATH_A, rf_freq));
             CHECKERR (rssmu_SetOutputState (io, RSSMU_PATH_A, state));
             rssmu_close(io);

             SetWaitCursor (0);
             SetCtrlAttribute (panelHandle, PANEL_APPLY, ATTR_DIMMED, 0);
             SetCtrlAttribute (panelHandle, PANEL_Exit, ATTR_DIMMED, 0);
        
            break;
    }
    return 0;
}

/*===========================================================================*/ 
/* Local Functions Definitions                                               */
/*===========================================================================*/

/*===========================================================================*/
/* Function: Initialize                                                      */
/* Purpose:  This function initializes the instrument, performs ID Query     */
/*           and Reset                                                       */
/*===========================================================================*/
int initialize (void)
{
    ViChar      instr_desc[256];
    ViBoolean   id;
    ViBoolean   rst;
    

    SetWaitCursor(0);
    GetCtrlVal (panelHandle, PANEL_INSTR_DESC, instr_desc);
    GetCtrlVal (panelHandle, PANEL_IDQUERY, &id);
    GetCtrlVal (panelHandle, PANEL_RESET, &rst);
    SetWaitCursor(1);
    if ((error = rssmu_init (instr_desc, id, rst, &io)) < 0 )
    {
        rssmu_error_message(VI_NULL, error, error_message);
        sprintf (error_buffer, "Initialization Error:\n\n%s\n\nCheck your "
                "connections and make sure you have the right instrument descriptor.",
                 error_message);
        MessagePopup ("ERROR!", error_buffer);
        SetWaitCursor(0);
        return(-1);   
    }

    viSetAttribute (io, VI_ATTR_TMO_VALUE, 10000);
    SetWaitCursor(0);
    return 0;
}
